unit PPreview;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, Printing,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Tabs, INIFILES, graphics,printers;

const
  maxtabs = 10; {presume maximum 10 col in a table print}
  NA = -32768; { n/a value }
  BOXLINENONE = 0;
  BOXLINELEFT = 1;
  BOXLINERIGHT = 2;
  BOXLINELEFTRIGHT = 3;
  BOXLINETOP = 4;
  BOXLINELEFTTOP = 5;
  BOXLINERIGHTTOP = 6;
  BOXLINENOBOTTOM = 7;
  BOXLINEBOTTOM = 8;
  BOXLINELEFTBOTTOM = 9;
  BOXLINERIGHTBOTTOM = 10;
  BOXLINENOTOP = 11;
  BOXLINETOPBOTTOM = 12;
  BOXLINENORIGHT = 13;
  BOXLINENOLEFT = 14;
  BOXLINEALL = 15;

  BOXLINEHORIZ = BOXLINETOPBOTTOM;
  BOXLINEVERT = BOXLINELEFTRIGHT;

type
  TPrintJustify = (pjCenter,pjLeft,pjRight,pjBlock);
  TOrientation = (portrait,lanscape);
  TPreview = class; {forward}
  TDevCaps = function(DeviceName, Port: PChar; index: Word;
    OutPut: PChar; DevMode: PDevMode): Longint;

  tabrec = record
    X,left,width : double;
    Types : TPrintJustify;
    BoxType : byte;
    Shade : byte;
   end;

  TMtabs = array[1..maxtabs] of tabrec;

  TPPreviewForm = class(TForm)
    Panel2    : TPanel;
    Label1    : TLabel;
    Label2    : TLabel;
    PageLabel : TLabel;
    ScrollBox1: TScrollBox;
    ZoomEdit: TEdit;
    PageEdit: TEdit;

    PrintDialog1: TPrintDialog;
    TabSet1: TTabSet;

    SBZoomIn        : TSpeedButton;
    SBZoomOut       : TSpeedButton;
    SBPrevPage      : TSpeedButton;
    SBNextPage      : TSpeedButton;
    SBDone          : TSpeedButton;
    SBPrint         : TSpeedButton;
    SBZoomPageWidth : TSpeedButton;
    SBZoomPage      : TSpeedButton;
    SBFirstPage     : TSpeedButton;
    SBLastPage      : TSpeedButton;
    Image1: TImage;

    procedure SBZoomInClick(Sender: TObject);
    procedure SBZoomOutClick(Sender: TObject);
    procedure SBPrevPageClick(Sender: TObject);
    procedure SBNextPageClick(Sender: TObject);
    procedure SBPrintClick(Sender: TObject);
    procedure SBDoneClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SBZoomPageWidthClick(Sender: TObject);
    procedure SBZoomPageClick(Sender: TObject);
    procedure FilePreview1PageChange(Sender: TObject);
    procedure FilePreview1ZoomChange(Sender: TObject);
    procedure PageEditExit(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
                          Shift: TShiftState);
    procedure PageEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure TabSet1Click(Sender: TObject);
    procedure SBFirstPageClick(Sender: TObject);
    procedure SBLastPageClick(Sender: TObject);
    procedure ZoomEditChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Image1DblClick(Sender: TObject);

  private
    Prevw : TPreview;
    FONTHEIGHt: integer;
    ZoomFactor : real;
    zoom : integer;
    screenScaleX,screenScaleY:real;
    Maxx,Maxy : longint;
    OffsetX,OffsetY: longint;
    twipX,twipY: real;
    wherex, wherey: integer;
    Leftmargin, topmargin, rightmargin,bottommargin: Integer;
    rightjustify, center, bold, italic: boolean;
    pages:array[1..99] of integer;
    totalpages : integer;
    FOnPrintFooter  : TnotifyEvent;
    Tabct : integer;
    Mtabs : TMtabs;

    procedure SetSize( pixelsperinch : longint );
    procedure ResetPrinter;
    Procedure PrintTxt (Txt: string);
    procedure processtxt(Txt: string);
    Function checkpage(tmp:string):integer;
    function ConvX(x : integer) : integer;
    function ConvY(y : integer) : integer;
    procedure DrawMargins;
    procedure ExecuteCustom(frm,tto,copies:Integer);
    procedure dosetup;
    procedure ClearTabs;
    PROCEDURE SETTAB(S:STRING);
  public
    resox,resoy : real;
    Mcanvas : tcanvas;
    currentpage : integer;
    Showmargin : boolean;
    ToPrinter: Boolean;
    Aborted : boolean;
    Procedure Home;
    procedure Println(Text: string);
    procedure PrintCENTER(Text: string);
    procedure SetFont(f:string;s:integer);
    procedure clear;
    procedure printpage(pagenumber:integer);
    procedure PrintFooter(Text: string;Justify: TPrintJustify);
  end;

  TPreview = class(TComponent)
  private
       fPrintPreview   : TPPreviewForm;
       bitmap          : TBitmap;
       FPageCount      : Longint;
       FDrawMargins    : Boolean;
       PageNumber      : Longint;
       TMPSTR : STRING;
       Fonprint : TnotifyEvent;
       FonNewpage : TnotifyEvent;
       FonPrintApage : TnotifyEvent;
       FOnPrintHeader : TnotifyEvent;
       FOnPrintFooter  : TnotifyEvent;
       PageOriented : Torientation;
       Fpreview:boolean;
       Numbins : integer;
  Protected
   procedure getbins;
   procedure setbold(value:boolean);
   procedure setItalic(value:boolean);
   procedure setUnderline(value:boolean);
  Public
    Plist : TStrings;
		marginleft ,marginright ,margintop ,marginbottom :double;
    title : string[60];
    LinesLeft : INTEGER;
    currentpage: integer;
    Pageheight : integer;
    Pagewidth : integer;
    SectionBottom : real;
    Aborted : boolean;
    SectionLeft : double;
    Property Bold : boolean write setBold;
    Property Italic : boolean write setItalic;
    Property Underline : boolean write setUnderline;
    procedure home;
    Procedure Abort;
    procedure Print(Text: string);
    procedure Println(Text: string);
    procedure PrintCENTER(Text: string;n: reaL);
    procedure SetFont(f:string;s:integer);
    procedure CRLF;
    procedure Tab(LeftWidth: integer;
                            RightWidth: integer;
                            TopWidth: integer;
                            BottomWidth: integer;
                            ShadeOverride: integer);
    procedure SetTab(NewPos: double;
                     NewJustify: TPrintJustify;
                     NewWidth: double;
                     NewMargin: double;
                     NewLines: byte;
                     NewShade: byte);
    Procedure ClearTabs;
    constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
    Function Execute: boolean;
    procedure Newpage;
    procedure PrintFooter(Text: string; Justify: TPrintJustify);
    Function SUPPORTBIN(N:integer):boolean;
    Procedure Selectbin(n:integer);
   published
    property OnPrint :TnotifyEvent read Fonprint write Fonprint;
    property OnPrintAPage :TnotifyEvent read FOnPrintAPage write FOnPrintAPage;
    property OnNewPage :TnotifyEvent read FOnNewPage write FOnNewPage;
    property OnPrintHeader:TnotifyEvent read FOnPrintHeader write FOnPrintHeader;
    property OnPrintFooter:TnotifyEvent read FOnPrintFooter write FOnPrintFooter;
    Property PreviewMode: boolean read Fpreview write Fpreview default true;
    Property ShowMargin: Boolean read FDrawMargins write FDrawMargins default false;
    Property Orientation: Torientation read PageOriented write PageOriented default portrait;
  end;

procedure register;
implementation
{$R *.DFM}
const
     Ppixelsperinch = 584;
     PreviewSizeFull = 144;
     PreviewSizeThreeQuarters = 108;
     PreviewSizeHalf = 72;
     PreviewSizeQuarter = 36;
     spacing = 10;

procedure register;
begin
   RegisterComponents('Custom', [TPreview]);
end;
function Trim (S: string): string;
var
  ii: byte;
begin
   ii := 1;
   While (Length (S) >= ii) and
         (S [ii] = ' ') do
         inc (ii);
   S := copy (S, ii, 255);
   ii := Length (S);
   If ii = 0 then begin
      Trim := '';
      exit;
   end;
   While (S [ii] = ' ') and (ii > 0) do
         dec (ii);
   if ii > 0 then
    Trim := copy (S, 1, ii)
   else
    trim := '';
end;

procedure TPPreviewForm.home;
begin
 wherex := leftmargin;
 wherey := topmargin;
end;

procedure TPPreviewForm.ExecuteCustom(frm,tto,copies:Integer);
var f,i : integer;
   oldx, oldy : real;
   oldzoom : integer;
   prt : TfPrinting;

begin
aborted := false;
oldx := resox;
resox := 1; {0.85}
oldy := resoy;
resoy := 1;
oldzoom := zoom;
zoom := Ppixelsperinch;
if frm = 0 then
  frm := 1;
if tto = 0 then
  tto := totalpages;
printer.begindoc;
printer.title := tpreview(prevw).title;
mcanvas := printer.canvas;
mcanvas.font := font;
prt := TfPrinting.Create(self);
prt.SetMax( tto );
prt.Show;
Application.ProcessMessages;
for f := 1 to copies do
begin
I := frm;
while ( i <= tto ) and ( not aborted) do
    begin
    prt.SetProgress( i );
    Application.ProcessMessages;
    printpage(i);
    if i < tto then
     printer.newpage;
    inc(i);
    end;
end; (* there is a way to print! *)
prt.Hide;
prt.Release;
printer.enddoc;
resox := oldx;
resoy := oldy;
zoom := oldzoom;
mcanvas:= image1.canvas;
end;


procedure TPPreviewForm.PrintFooter(Text: string;
                                    Justify: TPrintJustify);

begin { TPreviewForm.}
if not toprinter then
mcanvas.font.size := round(ScrollBox1.font.size * resox)
ELSE
BEGIN
 mcanvas.font.STYLE := [];
 mcanvas.font.SIZE := 9;
END;
case Justify of
  pjCenter:mcanvas.textout((mcanvas.cliprect.right - mcanvas.textwidth(text)) div 2
            , mcanvas.cliprect.bottom - bottommargin -  mcanvas.textheight('H') -10,Text);
  pjLeft:mcanvas.textout(leftmargin, mcanvas.cliprect.bottom - bottommargin -mcanvas.textheight('H') -10,Text);
  pjRight:mcanvas.textout(mcanvas.cliprect.right - leftmargin - mcanvas.textwidth(text) -10
            , mcanvas.cliprect.bottom - bottommargin - mcanvas.textheight('H') -10,Text);
end;

end;  { PrintFooter }

procedure TPPreviewForm.DrawMargins;
var
   col : TColor;
begin
if toprinter then exit;
   col := mcanvas.Pen.Color;
   mcanvas.Pen.Color := clGray;
   mcanvas.Pen.Style := psDash;
   MCanvas.Rectangle( ConvX(0) -2, ConvY(0) - 2, maxx - ConvX(0) +2, maxy - ConvY(0)+2 );
   mcanvas.Pen.Color := col;
   mcanvas.Pen.Style := psSolid;
end;
function TPPreviewForm.ConvX(x : integer) : integer;
var
   a : double;
begin
     result := Round(twipX * (x+offsetx));
end;

function TPPreviewForm.ConvY;
begin
     result := Round(twipY * (y+offsety));
end;

procedure TPPreviewForm.SBZoomInClick(Sender: TObject);
begin
if zoom < PreviewSizeFull then
 zoom := zoom + PreviewSizeQuarter;
 setsize(zoom);
 ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100);
 printpage(currentpage);
 invalidate;
end;

procedure TPPreviewForm.SBZoomOutClick(Sender: TObject);
begin
if zoom > PreviewSizeQuarter then
 zoom := zoom - PreviewSizeQuarter;
 setsize(zoom);
 ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100);
 printpage(currentpage);
 invalidate;
end;

FUNCTION GET_VALUE_STR(C:CHAR;VAR S:STRING):STRING;
BEGIN
   GET_VALUE_STR := '';
   IF POS(C,S) > 0 THEN
   BEGIN
      DELETE(S,1,POS(C,S));
      GET_VALUE_STR := COPY(S,1,POS('"',S) -1);
      DELETE(S,1,POS(C,S));
   END;
END;

procedure TPPreviewForm.SBPrevPageClick(Sender: TObject);
begin
if currentpage - 1 > 0 then
  begin
   dec(currentpage);
   tabset1.tabindex := currentpage -1;
   printpage(currentpage);
  end;
end;

procedure TPPreviewForm.SBNextPageClick(Sender: TObject);
begin
if currentpage + 1 <= totalpages then
  begin
   inc(currentpage);
   tabset1.tabindex := currentpage -1;
   printpage(currentpage);
  end;
end;

procedure TPPreviewForm.SBPrintClick(Sender: TObject);
begin
SBPrint.enabled := false;
  with PrintDialog1 do
    begin
      If totalPages <> 0 then
        begin
          Copies := 1;
          MinPage := 1;
          MaxPage := totalPages;
          FromPage := 1;
          ToPage := MaxPage;
          if MaxPage <> 1 then
            Options := [poPageNums]
          else
            Options := [];
        end;
    end;
  if PrintDialog1.Execute then
    begin
      with PrintDialog1 do
        begin
          toprinter := true;
          if ToPage > totalPages then
             ToPage := totalPages;
          if PrintRange = prPageNums then
            ExecuteCustom(FromPage, ToPage, Copies)
          else
            ExecuteCustom(0, 0, Copies);
        end;
    end;
toprinter := false;
SBPrint.enabled := true;
end;

procedure TPPreviewForm.SBDoneClick(Sender: TObject);
begin
  ModalResult := mrOk;
  Close;
end;

procedure TPPreviewForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TPPreviewForm.SBZoomPageWidthClick(Sender: TObject);
begin
Zoom := PreviewSizeThreeQuarters;
setsize(Zoom);
ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100);
printpage(currentpage);
invalidate;
end;

procedure TPPreviewForm.SBZoomPageClick(Sender: TObject);
begin
Zoom := PreviewSizeHalf;
setsize(Zoom);
ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100);
printpage(currentpage);
invalidate;
end;

procedure TPPreviewForm.FilePreview1PageChange(Sender: TObject);
begin
      PageEdit.Text := IntToStr(CurrentPage);
      PageLabel.Caption := 'Page ' + IntToStr(CurrentPage) +
                           ' of ' + IntToStr(totalPages);
      SBPrevPage.Enabled := CurrentPage > 1;
      SBFirstPage.Enabled := SBPrevPage.Enabled;
      SBNextPage.Enabled := currentpage < totalpages;
      SBLastPage.Enabled := SBNextPage.Enabled;
end;

procedure TPPreviewForm.FilePreview1ZoomChange(Sender: TObject);
begin
ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100)
end;

procedure TPPreviewForm.PageEditExit(Sender: TObject);
var
  I1: integer;
  ErrCode: integer;
begin
  Val(PageEdit.Text,I1,ErrCode);
  If (ErrCode = 0) and (CurrentPage <> I1) then
     if i1 <= totalpages then
       begin
       currentpage := I1;
       PrintPage(I1);
      end;
end;

procedure TPPreviewForm.FormKeyDown(Sender: TObject; var Key: Word;
                                   Shift: TShiftState);
var
  I1: integer;
begin
  Case Key of
    vk_Up:
      With ScrollBox1.VertScrollBar do
        begin
          I1 := Position - Increment;
          If I1 < 0 then I1 := 0;
          Position := I1;
        end;

    vk_Down:
      With ScrollBox1.VertScrollBar do
        begin
          I1 := Position + Increment;
          If I1 >= Range then I1 := Range - 1;
          Position := I1;
        end;

    vk_Prior:
      begin
        With ScrollBox1.VertScrollBar do
          Position := Position - ScrollBox1.Height;
      end;

    vk_Next:
      begin
        With ScrollBox1.VertScrollBar do
          Position := Position + ScrollBox1.Height;
      end;

    vk_Left : begin end;
    vk_Right: begin end;

    vk_Escape:
      begin
        Close;
        Key := 0;
      end;

  end; { case }
end;

procedure TPPreviewForm.PageEditKeyPress(Sender: TObject; var Key: Char);
begin
  If Key = #13 then PageEditExit(Sender);
end;

procedure TPPreviewForm.FormCreate(Sender: TObject);
VAR T: TINIFILE;
		N : LONGINT;
begin
  ClearTabs;
  Leftmargin:= 0;
  rightmargin := 0;
  bottommargin :=0;
  Topmargin := 0;
  currentpage := 1;
	ToPrinter := false;
  mcanvas :=Image1.canvas;
	T := TINIFILE.CREATE('PREVIEW.INI');
	N := T.READINTEGER('SCREEN','ZOOM',100);
	T.FREE;
 wherex:= 0;
 wherey:= 0;
 ZoomFactor:= 1;
 zoom:= PreviewSizeHalf;
 Windowstate := wsMaximized;
 SetSize(zoom);
 mcanvas.pen.color := clblack;
 mcanvas.font := font;
 ScrollBox1.font := font;
 FONTHEIGHt := mcanvas.TEXtHEIGHT('H');
 ZoomEdit.Text := FLOATTOSTR((Zoom /PreviewSizeFull) * 100);
 resetprinter;
 wherex := leftmargin +1;
 wherey := topmargin +1;
end;

procedure TPPreviewForm.TabSet1Click(Sender: TObject);
begin
	with TabSet1 do
		begin
			If (TabIndex >= 0) and
				 (CurrentPage <> TabIndex + 1) then
         begin
          currentpage :=  TabIndex + 1;
				  PrintPage(currentpage);
         end;
		end; { with }
end;

procedure TPPreviewForm.SBFirstPageClick(Sender: TObject);
begin
currentpage := 1;
tabset1.tabindex := currentpage -1;
printpage(currentpaGE);
end;

procedure TPPreviewForm.SBLastPageClick(Sender: TObject);
begin
currentpage := totalpages;
tabset1.tabindex := currentpage -1;
printpage(currentpaGE);
end;

procedure TPPreviewForm.ZoomEditChange(Sender: TObject);
VAR T : TINIFILE;
		N : DOUBLE;
begin
	N := STRTOFLOAT(ZoomEdit.TEXT);
  ZoomFactor :=n;
	T := TINIFILE.CREATE('PREVIEW.INI');
	T.WRITEINTEGER('SCREEN','ZOOM',ROUND(N));
	T.FREE;
end;

procedure TPPreviewForm.ResetPrinter;
var
    PrintOffset: TPoint;
    PageSize: TPoint;
    EscapeFunc: word;
    DevModeHandle: word;
    DevMode: PDevMode;
    Handle: HDC;
    FXPixPerMM,FYPixPerMM: real;
    FLeftWaste,FRightWaste, FTopWaste ,FbottomWaste:integer;
begin { ResetPrinter }
    FLeftWaste := 0;
    FRightWaste := 0;
    FTopWaste := 0;
    FbottomWaste := 0;
    { Get current printer settings }
      EscapeFunc := GETPRINTINGOFFSET;
      Printer.Canvas.MoveTo(0,0);
      FXPixPerMM := GetDeviceCaps(Printer.Handle,LOGPIXELSX) / 25.4;
      FYPixPerMM := GetDeviceCaps(Printer.Handle,LOGPIXELSY) / 25.4;
      If Escape(Printer.Handle,QUERYESCSUPPORT,SizeOf(EscapeFunc),@EscapeFunc, nil) <> 0 then
      begin
         EscapeFunc := GETPHYSPAGESIZE;
         If Escape(Printer.Handle,QUERYESCSUPPORT,SizeOf(EscapeFunc), @EscapeFunc,nil) <> 0 then
         begin
           EscapeFunc := GETPRINTINGOFFSET;
           Escape( Printer.Handle,GETPRINTINGOFFSET,SizeOf(EscapeFunc),
                   @EscapeFunc, @PrintOffset);
           EscapeFunc := GETPHYSPAGESIZE;
           Escape( Printer.Handle, GETPHYSPAGESIZE, SizeOf(EscapeFunc),
                   @EscapeFunc,@PageSize);
           FLeftWaste  := round (PrintOffset.X / FXPixPerMM);
           FRightWaste := round ((PageSize.X - (GetDeviceCaps(Printer.Handle,
                                 HORZRES) + PrintOffset.X)) / FXPixPerMM);
           FTopWaste   := round (PrintOffset.Y / FYPixPerMM);
           FBottomWaste:= round ((PageSize.Y - (GetDeviceCaps(Printer.Handle,
                                 VERTRES) + PrintOffset.Y)) / FYPixPerMM);
         end;
     end;
{    end;}
{    LeftMargin   := FLeftWaste;
    RightMargin  := FrightWaste;
    TopMargin    := FTopWaste;
    BottomMargin := FbottomWaste;}
end;


procedure TPPreviewForm.SetSize( pixelsperinch : longint );
var
   pixelsperinchx  : longint;
   pixelsperinchy  : longint;
   fullHeight,
   fullWidth       : longint;
   physsize        : TPOINT;
begin
   RESOX := ((PreviewSizeQuarter + pixelsperinch) /PreviewSizeFull);
   RESOY := ( pixelsperinch /PreviewSizeFull);
   Escape( printer.handle, GETPHYSPAGESIZE, 0, nil, @physsize );

   pixelsperinchx := GetDeviceCaps( printer.handle, LOGPIXELSX );
   pixelsperinchy := GetDeviceCaps( printer.handle, LOGPIXELSY );
{   fullHeight := printer.pageheight;}
   (*fPrintPreview.Mcanvas.PixelsPerInchDevice := pixelsperinchx;*)
   fullHeight := Round((physsize.y  * pixelsperinch ) / pixelsperinchy );
   fullWidth  := Round((physsize.x * pixelsperinch ) / pixelsperinchx);

   OffsetX := ((physsize.x - printer.pagewidth) div 2);
   OffSetX := (OffsetX * 1440) div pixelsperinchx;
   OffSetY := ((physsize.y - printer.pageheight) div 2);
   OffSetY := (OffsetY * 1440) div pixelsperinchy;

   screenScaleX := ( fullWidth / physsize.x );
   screenScaleY := ( fullHeight / physsize.y );

   maxX  := fullWidth;
   maxY  := fullHeight;
   twipX := pixelsperinch / 1440;
   twipY := pixelsperinch / 1440;

   Image1.Picture.bitmap.Height := fullHeight;
   Image1.height :=fullHeight;
   Image1.Picture.bitmap.Width  := fullWidth;
   Image1.width :=fullWidth;
{   scrollbox1.vertscrollbar.range := Image1.height + 10;}
   mcanvas.brush.color := clwhite;
   mcanvas.rectangle(0,0,mcanvas.cliprect.right,mcanvas.cliprect.bottom);
   if screen.width > fullWidth then
     Image1.left := (screen.width - image1.picture.bitmap.width) div 2
   else
    Image1.left := 0;
    LeftMargin   := convx(0);
    TopMargin    := convy(0);
end;

Procedure TPPreviewForm.PrintTxt (Txt: string);
var tmpx : longint;
begin
if not toprinter then
   mcanvas.font.size := round(ScrollBox1.font.size * resox);
tmpx := wherex;
   if rightjustify then
   begin
       wherex := round(wherex {* resox});
       IF wherex > MCANVAS.textwidth(TXT) THEN
       BEGIN
         wherex := wherex - MCANVAS.textwidth(TXT);
         wherex := round(wherex {/ resox});
         IF wherex < leftmargin THEN
          wherex :=leftmargin;
       END
       ELSE
         wherex :=leftmargin;
       RIGHTJUSTIFY := FALSE;
   end
   else
      IF CENTER THEN
      BEGIN
         wherex := MCANVAS.textwidth(TXT);
         wherex:= (mcanvas.cliprect.right - MCANVAS.textwidth(TXT) ) DIV 2;
         wherex := round(wherex{ / resox});
         center := false;
      END;
   with Mcanvas do
      begin
         TextOut (round(wherex {* resox}), wherey, Txt);
         tmpx := TextWidth (Txt);
         wherex := wherex + round( tmpx {/ resox})
      end;
end;

procedure TPPreviewForm.printcenter(text:string);
begin
center := true;
println(text);
end;

procedure TPPreviewForm.printpage(pagenumber:integer);
var i : longint;
    from,tto : longint;
    tmp : string;
begin
if showmargin then
 DrawMargins
else
 clear;
wherex := leftmargin;
wherey:= topmargin;
if pagenumber  = 1  then
begin
 from := 1;
 if totalpages = 1 then
  tto := prevw.plist.count - 1
 else
  tto := pages[pagenumber + 1];
end
else
if pagenumber > 0 then
if pagenumber <= totalpages then
begin
 from := pages[pagenumber];
 tto := pages[pagenumber + 1];
end;
if (tto = 0) or (tto >= prevw.plist.count) then
   tto := prevw.plist.count - 1;
for i :=  from to  tto do
 begin
  tmp := prevw.plist.strings[i];
  IF POS('<TAB>',tmp) = 1 THEN
  BEGIN
    SETTAB(TMP);
    CONTINUE;
  END;
  IF POS('<CLEARTAB>',tmp) = 1 THEN
  BEGIN
    CLEARTABS;
    CONTINUE;
  END;
  println(tmp);
end;
if assigned(FonPrintfooter) then
  FOnPrintfooter(self);
FilePreview1PageChange(self);
end;

FUNCTION GETVALUE(C,S:STRING):INTEGER;
VAR
   N: INTEGER;
   M: INTEGER;
   S1 :STRING;
BEGIN
   S1 := '';
   GETVALUE := -1;
   N := POS(C,S);
   IF N > 0 THEN
   BEGIN
      S1 := COPY(S,N+LENGTH(C),3);
      WHILE POS(' ',S1) > 1 DO
         DELETE(S1,POS(' ',S1),1);
      FOR M:= 1 TO LENGTH(S1) DO
         IF NOT (S1[M] IN ['0'..'9']) THEN
         BEGIN
            DELETE(S1,M,3);
            BREAK;
         END;
      VAL(S1,M,N);
      IF N = 0 THEN
         GETVALUE := M;
     END;
END;


FUNCTION GET_FONT_SIZE(FN:STRING;N:BYTE):BYTE;
BEGIN
   GET_FONT_SIZE := n ;
END;

procedure TPPreviewForm.processtxt;
var Attrib : string[80];
   Fraze, s : string;
   p1, p2: byte;
PROCEDURE SetFStyle(C:CHAR;B:BOOLEAN);
VAR FStyle: TFontStyles;
   FS: integer;
BEGIN
   FStyle := [];
   FS := 0;
   IF fsBold IN  Mcanvas.font.style THEN
      FS := 1;
   IF fsItalic IN  Mcanvas.font.style THEN
      FS := FS OR 2;
   IF fsUnderline IN  Mcanvas.font.style THEN
      FS := FS OR 4;
   case c of
      'B':IF b then
         FS := fs or 1
      else
         fs := fs and 6;
      'I': IF b then
         FS := fs or 2
      else
         fs := fs and 5;
      'U':IF b then
         FS := fs or 4
      else
         fs := fs and 3;
   end;
   if (FS and 1) = $01 then FStyle := FStyle + [fsBold];
   if (FS and $02) = $02 then FStyle := FStyle + [fsItalic];
   if (FS and $04) = $04 then FStyle := FStyle + [fsUnderline];
    Mcanvas.Font.Style := FStyle;
END;

PROCEDURE Set_Font_Attr(var TXT : STRING);
VAR TM : STRING;
   CPOS, SPOS: BYTE;
   FNAME : STRING[20];
BEGIN
   TM := TXT;
   IF POS('FACE',TM) > 0 THEN
      FNAME := GET_VALUE_STR('"',TM);
   Mcanvas.FONT.name := FNAME;
   SPOS := POS('SIZE',TM);
   CPOS := POS('COLOR',TM);
   IF SPOS > 0 THEN
   begin
       Mcanvas.FONT.SIZE := GET_FONT_SIZE(FNAME,GETVALUE('=',TM));
       ScrollBox1.FONT.SIZE := Mcanvas.FONT.SIZE;
   end;
END;
procedure SetFontAttr;
var
   A: integer;
   FntA: string;
begin
   repeat
      A := pos (';', Attrib);
      if A = 0 then begin
         FntA := UPPERCASE(Attrib);
         Attrib := '';
      end
      else
      begin
         FntA := copy (Attrib, 1, A - 1);
         delete (Attrib, 1, A);
      end;
      if length(FntA) > 0 then
      try
         case Upcase(FntA[1]) of
           '/': begin
                  CASE FntA[2] OF
                   'B','I': begin
                        SetFStyle( FntA[2], FALSE);
                        bold := false;
                     end;
                   'U': SetFStyle( FntA[2], FALSE);
                   'C': IF FntA[3] = 'E' THEN CENTER := FALSE;
                   'F': begin
                         Mcanvas.Font.name := Font.name;
                         Mcanvas.Font.size := Font.size;
                         Mcanvas.Font.color := Font.color;
                       {  SPACING := FLineSpacing;}
                       end;
                    END; {CASE /}
                  end;
           'C': begin
                  if FntA = 'CENTER' THEN CENTER := TRUE;
                 end ;
           'B': If length(FntA) = 1 then
                  begin
                     SetFStyle( FntA[1], true);
                     bold := true;
                  end;

           'F':  Mcanvas.Font.Name := copy (FntA, 2, 30);
           'I': begin
                  If length(FntA) = 1 then
                     SetFStyle( FntA[1], true)
                  end;
           'S': if FntA[2] in['0'..'9'] then
                begin
                Mcanvas.Font.Size := StrToInt (copy (FntA, 2, 3));
                ScrollBox1.FONT.SIZE := Mcanvas.Font.Size;
                end;
           'X': begin
              if toprinter then
                wherex := StrToInt (copy (FntA, 2, 4) ) + leftmargin
               else
                wherex := ROUND(StrToInt (copy (FntA, 2, 4))) {+ leftmargin };
               if wherex < leftmargin then wherex := leftmargin
               end;
           'U': begin
                  SetFStyle( FntA[1], true);
                  end;
               END;
         except
            MessageBeep(1);
         end;
      until Attrib = '';
   end;
VAR BLANKLINE : BOOLEAN;
begin
    BLANKLINE := false;
      S := txt;
      Fraze := '';
      if pos('<PAGE',S) = 1 THEN
      begin
         exit;
      END;
      
      IF (pos('>',s) >= pos('<',s)) then
      BEGIN
      repeat
         {FONT SIZE, COLOR}
         P1 := pos('<FONT FACE=',s);
         iF P1 = 1 THEN
         BEGIN
            SET_FONT_ATTR(S);
            mcanvas.font.size := round(mcanvas.font.size * resox);
            ScrollBox1. font.size := mcanvas.font.size;
            EXIT;
         END;
         P1 := pos ('<', S);
         if P1 = 0 then
         begin
            Fraze := S;
            PrintTxt (Fraze);
            break;
         end;
         P2 := pos ('>', S);
         Attrib := copy (S, P1 + 1, P2 - P1 - 1);
         if P1 > 1 then
         begin
            Fraze := copy (S, 1, P1 - 1);
            PrintTxt (Fraze);
         end;
         Delete (S, 1, P2);
         SetFontAttr;
      until Length (S) = 0;
      END;
 IF (NOT BLANKLINE)  THEN
  BEGIN
  IF TOPRINTER THEN
   wherey := whereY + MCanvas.TextHeight ('H') + Round (spacing * resoy)
  ELSE
   begin
    if resoy > 0.5 then
      wherey := whereY + MCanvas.TextHeight ('H') -  Round (spacing * (resoy - 0.6))
     else
      wherey := whereY + MCanvas.TextHeight ('H') - Round (spacing * (resoy - 0.25));
   end;
  whereX := leftmargin;
  END;

end;

function pat(ch: char; n : byte) : string;
var s : string;
    i : byte;
begin
s := '';
for i := 1 to n do
 s := s + ch;
pat := s;
end;

function ShadeToColor(ShadeColor: TColor;
                        ShadePercent: byte): TColor;

  var
    Red: byte;
    Blue: byte;
    Green: byte;
    RGBVal: longint;

  begin { ShadeToColor }
    If ShadePercent > 100 then begin
      ShadePercent := 100;
    end; { if }
    RGBVal := ColorToRGB(ShadeColor);
    Red := ((255 - GetRValue(RGBVal)) * (100 - ShadePercent)) div 100 +
     GetRValue(RGBVal);
    Green := ((255 - GetGValue(RGBVal)) * (100 - ShadePercent)) div 100 +
     GetGValue(RGBVal);
    Blue := GetBValue(RGBVal);
    Blue := ((255 - Blue) * (100 - ShadePercent)) div 100 + Blue;

    Result := RGB(Red,Green,Blue);
  end;  { ShadeToColor }


procedure TPPreviewForm.println(text:string);
var s : string;
    i : integer;
    n : byte;
    temp : integer;
    space: integer;
    tzoom : integer;
begin
n:= 0;
s:= text;
i := pos(#9,s);
if toprinter then
 begin
   tzoom := zoom;
   mcanvas.pen.width := 3;
   space := fontheight div 6;
 end
else
 begin
   tzoom := zoom;
   mcanvas.pen.width := 1;
{   if zoom > 75 then
      space := (fontheight div 8)
   else}
      space := (fontheight div 8);
   if zoom > 108 then
    inc(wherey, 1);
 end;

if i > 0 then
 inc(wherey,space + 1);

while (i > 0) and (n <= tabct) do
 begin
  inc(n);
  mcanvas.brush.STYLE := Bsclear;
  IF mtabs[n].shade > 1 THEN
  BEGIN
   mcanvas.brush.STYLE := BSSOLID;
   mcanvas.brush.color := ShadeToColor(CLNAVY,mtabs[n].shade);
  END;
  case mtabs[n].BoxType of
    BOXLINENONE:;
    BOXLINETop :begin
                mcanvas.moveto(round(mtabs[n].X * tzoom) ,
                       wherey - round(space * resoy));
                mcanvas.lineto(round(mtabs[n].X * tzoom + mtabs[n].width * tzoom),
                       wherey - round(space * resoy));
                end;
    BOXLINEALL: mcanvas.rectangle(round((mtabs[n].X * tzoom)) ,
       wherey - round(space * resoy),
       round(mtabs[n].X * tzoom + mtabs[n].width * tzoom),
       wherey + MCANVAS.TEXTHEIGHT('H') + round(({fontheight + }space) * resoy));
  end;
  delete(s,1,i);
  i := pos(#9,s);
 end;
 mcanvas.brush.style := Bsclear;
 n:= 0;
 i :=pos(#9,text);
 while  i > 0 do
  begin
    inc(n);
    temp := round(mtabs[n].X * tzoom);
    delete(text,i,1);
    if pos(#9,text) > 0 then
     s:= copy(text,i,pos(#9,text) -I)
    else
     s:= copy(text,i,length(text));

    case mtabs[n].types of
     pjCenter: temp := round((((mtabs[n].X *2) + mtabs[n].width) * tzoom)
        - mcanvas.textwidth(s)) div 2;
     pjLeft:temp := round(mtabs[n].X * tzoom) + mcanvas.textwidth('H') DIV 5;
     pjRight:temp := round(((mtabs[n].X + mtabs[n].width) * tzoom)
        - mcanvas.textwidth(s)) - mcanvas.textwidth('H');
    end;
    if n < tabct then
    if (temp + mcanvas.textwidth(s)) > (mtabs[n+1].X * tzoom) then
        temp := round(mtabs[n].X * tzoom);
    insert('<X' + inttostr(temp) + '>',text,i);
    i :=pos(#9,text);
  end;
 processtxt(text);
end;

procedure TPPreviewForm.clear;
begin
if not toprinter then
begin
   mcanvas.brush.color := clwhite;
   mcanvas.rectangle(0,0,mcanvas.cliprect.right,mcanvas.cliprect.bottom);
{   tabset1.tabs.clear;}
end;
end;

procedure TPPreviewForm.SetFont(f:string;s:integer);
begin
 mcanvas.font.name := f;
 mcanvas.font.size := s;
 scrollbox1.font.size := s;
end;

function TPPreviewForm.checkpage;
 var ct: integer;
     n: integer;
begin
totalpages := byte(tmp[1]);
ct := 1;
pages[ct] := 1;
n:= pos(';',tmp);
if n > 0 then
 delete(tmp,1,n);
n:= pos(',',tmp);
while n > 0 do
 begin
  inc(ct);
  pages[ct] := strtoint(copy(tmp,1,n-1));
  delete(tmp,1,n);
  n:= pos(',',tmp);
 end;
checkpage := ct;
end;

procedure TPPreviewForm.FormShow(Sender: TObject);
var m: integer;
begin
tabset1.tabs.clear;
if totalpages > 1 then
begin
for m := 1 to totalpages do
   tabset1.tabs.add(inttostr(m));
   tabset1.tabindex := 0;
end
else tabset1.visible := false;
Zoom := PreviewSizeHalf;
setsize(Zoom);
printpage(currentpage);
SBZoomPageClick(sender);
end;

procedure TPPreviewForm.dosetup;
begin
fillchar(pages,sizeof(pages),0);
checkpage(prevw.plist.strings[0]);
end;

procedure TPPreviewForm.SETTAB;
VAR
 NewPos: double;
 NewJustify: TPrintJustify;
 NewWidth: double;
 NewMargin: double;
 NewLines: byte;
 NewShade: byte;
 I,N: BYTE;
 TMP : STRING;
BEGIN
N:= 0;
DELETE(S,1,5);
I:= POS(',',S);
WHILE I > 0 DO
 BEGIN
  INC(N);
  TMP := COPY(S,1,I - 1);
  DELETE(S,1,I);
  CASE N OF
   1: newpos := STRTOFLOAT(TMP);
   2: NewJustify := TPrintJustify(BYTE(TMP[1]));
   3: NewWidth := STRTOFLOAT(TMP);
   4: NewMargin := STRTOFLOAT(TMP);
   5: NewLines := BYTE(TMP[1]);
   6: NewShade := BYTE(TMP[1]);
  END;
 I:= POS(',',S);
 END;
inc(tabct);
if newpos > 0 then
 Mtabs[tabct].x := newpos
else
 if tabct > 1 then
  Mtabs[tabct].x := Mtabs[tabct-1].width + Mtabs[tabct-1].x;
if newwidth > 0 then
 Mtabs[tabct].width := newwidth
else
 if tabct > 1 then
  Mtabs[tabct].width :=   Mtabs[tabct -1].width;
Mtabs[tabct].Types :=  NewJustify;
Mtabs[tabct].left :=  round(NewMargin);
Mtabs[tabct].shade := newshade;
Mtabs[tabct].BoxType := NewLines;
END;

procedure TPPreviewForm.ClearTabs;
BEGIN
fillchar(Mtabs,sizeof(TMtabs),#0);
tabct := 0;
END;

{----------------------tpreview--------------------------}

Procedure TPreview.home;
begin
fPrintPreview.home;
end;

Destructor TPreview.destroy;
begin
plist.free;
fPrintPreview.free;
inherited destroy;
end;

Constructor TPreview.create;
begin
inherited create(AOwner);
plist := tstringlist.create;
SectionLeft := 0.2;
getbins;
currentpage := 1;
Fpreview := true;
Pagewidth := 2;
end;

function TPreview.execute;
var i : longint;
begin
Aborted := false;
linesleft := 60;
TMPSTR := '';
plist.clear;
{getbins;}
plist.add(#1';');
if assigned(FonprintHeader) then
 FonprintHeader(self);
if assigned(Fonprint) then
 Fonprint(self)
else if assigned(Fonprintapage) then
  FonprintApage(self);
if PageOriented = portrait then
 Printer.Orientation := poPortrait
else
  Printer.Orientation := poLandscape;

fPrintPreview := TPPreviewForm.create(nil);
fPrintPreview.toprinter := not Fpreview;

fPrintPreview.prevw := self;
fPrintPreview.showmargin := FDrawMargins;
fPrintPreview.FOnPrintFooter := FOnPrintFooter;
 fPrintPreview.dosetup;
if Fpreview then
  fPrintPreview.showmodal
else
  fPrintPreview.SBPrintClick(self);
Aborted := fPrintPreview.aborted;
end;


procedure TPreview.Print(Text: string);
begin
TMPSTR := TMPSTR + TEXT;
end;

procedure TPreview.CRLF;
begin
PRINTLN({'</F>' + }TMPSTR);
TMPSTR := '';
end;


procedure TPreview.PrintCENTER(Text: string;n: REAL);
begin
TMPSTR := '<CENTER>' + TEXT + '</CENTER>';
PRINTLN(TMPSTR);
end;

Procedure TPreview.Newpage;
var tmp:string;
begin
 tmp := plist[0];
 inc(byte(tmp[1]));
 tmp := tmp + inttostr(plist.count) + ',';
 plist[0] := tmp;
 plist.add('<PAGE>');
 LINEsLEFT := 60;
inc(currentpage);
If assigned(FonPrintheader) then
  FonprintHeader (self);
If assigned(fonnewpage) then
  FonNewpage(self);
end;

procedure TPreview.Println(Text: string);
begin
plist.add(text);
TMPSTR := '';
DEC(LINEsLEFT);
end;


procedure TPreview.SetFont(f:string;s:integer);
begin
plist.add('<FONT FACE="' + F + '" SIZE=' + INTTOSTR(S) + '>');
end;

procedure TPreview.abort;
begin
if fPrintPreview.showing then
  fPrintPreview.close;
fPrintPreview := nil;
end;

procedure TPreview.PrintFooter(Text: string;Justify: TPrintJustify);
begin
fPrintPreview.PrintFooter(Text,Justify);
end;

procedure TPPreviewForm.Image1DblClick(Sender: TObject);
begin
caption := inttostr(image1.width)+ ',' +inttostr(image1.height);
end;

procedure TPreview.SetTab(NewPos: double;
                     NewJustify: TPrintJustify;
                     NewWidth: double;
                     NewMargin: double;
                     NewLines: byte;
                     NewShade: byte);
begin
printLN('<TAB>' + FLOATTOSTR(NewPos) + ','
                + CHAR(BYTE(NewJustify)) + ','
                + FLOATTOSTR(NewWidth) + ','
                + FLOATTOSTR(NewMargin) + ','
                + CHAR(NewLines) + ','
                + CHAR(NewShade) + ',');
end;

procedure TPreview.Tab(LeftWidth: integer;
                            RightWidth: integer;
                            TopWidth: integer;
                            BottomWidth: integer;
                            ShadeOverride: integer);
begin
end;

procedure TPreview.setbold;
begin
if value then
 Print('<B>')
ELSE
 Print('</B>');
end;
procedure TPreview.setItalic;
begin
if value then
 Print('<I>')
ELSE
 Print('</I>');
end;
procedure TPreview.setUnderline;
begin
if value then
 Print('<U>')
ELSE
 Print('</U>');
end;

procedure TPreview.getbins;
var s: string;
   name,drive,port : string[20];
   Pname,Pdrive,Pport :array[0..40] of char;
   h: thandle;
   pp: pointer;
   n: integer;
begin
s:= '';
n:= 1;
with tinifile.create('win.ini') do
 begin
   s := readstring('Windows','device','');
  free;
 end;
name := copy(s,1,pos(',',s) -1);
delete(s,1, pos(',',s));
drive :=  copy(s,1,pos(',',s) -1);
delete(s,1, pos(',',s));
port := s;                  {GETDEVICECAPS}
STRPCOPY(PNAME,NAME);
STRPCOPY(PPORT,PORT);
strpcopy(pdrive,drive + '.drv');
h := loadlibrary(pdrive);
if h >= 32 then
 begin
  pp := GetProcAddress(H,'DEVICECAPABILITIES');
  if pp <> nil then
     n := TDevCaps(pp)(PNAME,PPORT,dc_bins,nil,nil);
  freelibrary(h);
 end;
numbins := n;
end;

procedure TPreview.ClearTabs;
begin
PRINTLN('<CLEARTAB>');
end;

function TPreview.SUPPORTBIN;
begin
SUPPORTBIN := (n <= numbins);
end;

procedure TPreview.Selectbin(n:integer);
var
    NewDevice,
    NewDriver,
    NewPort: array[0..40] of char;
    NewDeviceMode: THandle;
    DevMode: WinTypes.PDevMode;
begin
  Printer.GetPrinter(NewDevice,NewDriver,NewPort,NewDeviceMode);
  try
     DevMode := GlobalLock(NewDeviceMode);
     If (DevMode <> nil) and (DevMode^.dmDefaultSource <> N) then
     begin
          DevMode^.dmDefaultSource := N;
          DevMode^.dmFields := DevMode^.dmFields or DM_DEFAULTSOURCE;
          Printer.SetPrinter(NewDevice,NewDriver,NewPort,NewDeviceMode);
     end;
  finally
     GlobalUnlock(NewDeviceMode);
  end;
end;

end.
